/**
 * minigl.cpp
 * CS248 Assignment 1, Winter 2010
 * -------------------------------
 * Implement miniGL here.
 * Do not use any additional files
 */

#include <cstdio>
#include <cstdlib>
#include "minigl.h"

#include <vector>
#include <stack>
using namespace std;


/**
 * Classes
 */
class vector4
{
public:
	vector4(){}
	vector4(MGLfloat x, MGLfloat y) : x(x), y(y), z(0), w(1) {}
	vector4(MGLfloat x, MGLfloat y, MGLfloat z) : x(x), y(y), z(z), w(1) {}
	vector4(MGLfloat x, MGLfloat y, MGLfloat z, MGLfloat w) : x(x), y(y), z(z), w(w) {}
	vector4 operator+(vector4 v){ return vector4(x+v.x, y+v.y, z+v.z, w+v.w); }
	vector4 operator-(vector4 v){ return vector4(x-v.x, y-v.y, z-v.z, w-v.w); }
	vector4 operator*(MGLfloat s){ return vector4(s*x, s*y, s*z, s*w); }
	bool operator!=(vector4 v){ return (x!=v.x or y!=v.y or z!=v.z or w!=v.w); }	
	bool operator==(vector4 v){ return (x==v.x and y==v.y and z==v.z and w==v.w); }
public:
	MGLfloat x,y,z,w;
};

class matrix4x4
{
public:
	//implementation here
};

/**
 * Global variables
 */
stack<matrix4x4> modelview;
stack<matrix4x4> projection;
stack<matrix4x4>* stack_pointer = &modelview;

vector<vector4> vertexList; //holds vertices for the mglVertex3 calls

MGLpixel framebuffer[MGL_SCREEN_WIDTH][MGL_SCREEN_HEIGHT];
MGLfloat depthbuffer[MGL_SCREEN_WIDTH][MGL_SCREEN_HEIGHT];

/**
 * This is a helper function. It plots a white pixel on the frame buffer.
 */
void plot(unsigned int x, unsigned int y)
{
	//you may need to modify this function to suit your needs
	if(x >= MGL_SCREEN_WIDTH or y >= MGL_SCREEN_HEIGHT)
		return;

	MGLpixel color = 0;
	MGL_SET_RED(color, 255);
	MGL_SET_GREEN(color, 255);
	MGL_SET_BLUE(color, 255);
	framebuffer[x][y] = color;
}

/**
 * Initializes the miniGL system.
 */
void mglInit()
{
	//Setup the matrix stacks
	matrix4x4 identity;
	//you need to set the variable 'identity' to the identity matrix right here.
	modelview.push(identity);
	projection.push(identity);


	//clear the frame buffer and the depth buffer
	MGLpixel color = 0;
	for(unsigned x = 0; x < MGL_SCREEN_WIDTH; x++)
	{
		for(unsigned y = 0; y < MGL_SCREEN_HEIGHT; ++y)
		{
			framebuffer[x][y] = color;
			depthbuffer[x][y] = 1.0f;
		}
	}

	//Filling a quarter of the screen(to demonstrate the plot function)
	//This block of code is unneccessary. You should get rid of it.
	for(unsigned x = 0; x < MGL_SCREEN_WIDTH/2; x++)
		for(unsigned y = 0; y < MGL_SCREEN_HEIGHT/2; ++y)	
			plot(x,y);
}

/**
 * Standard macro to report errors
 */
inline void MGL_ERROR(const char* description) {
    printf("%s\n", description);
    exit(1);
}

/**
 * Read pixel data starting with the pixel at coordinates
 * (0, 0), up to (width,  height), into the array
 * pointed to by data.  The boundaries are lower-inclusive,
 * that is, a call with width = height = 1 would just read
 * the pixel at (0, 0).
 */
void mglReadPixels(MGLsize width,
                   MGLsize height,
                   MGLpixel *data)
{
	for(unsigned x = 0; x < width; x++)
		for(unsigned y = 0; y < height; ++y)
			data[y*width+x] = framebuffer[x][y];
}

/**
 * Start specifying the vertices for a group of primitives,
 * whose type is specified by the given mode.
 */
void mglBegin(MGLpoly_mode mode)
{
}

/**
 * Stop specifying the vertices for a group of primitives.
 */
void mglEnd()
{
}

/**
 * Specify a two-dimensional vertex; the x- and y-coordinates
 * are explicitly specified, while the z-coordinate is assumed
 * to be zero.  Must appear between calls to mglBegin() and
 * mglEnd().
 */
void mglVertex2(MGLfloat x,
                MGLfloat y)
{
}

/**
 * Specify a three-dimensional vertex.  Must appear between
 * calls to mglBegin() and mglEnd().
 */
void mglVertex3(MGLfloat x,
                MGLfloat y,
                MGLfloat z)
{
}

/**
 * Set the current matrix mode (modelview or projection).
 */
void mglMatrixMode(MGLmatrix_mode mode)
{
	if(mode == MGL_MODELVIEW)
		stack_pointer = &modelview;
	else if(mode == MGL_PROJECTION)
		stack_pointer = &projection;
}

/**
 * Push a copy of the current matrix onto the stack for the
 * current matrix mode.
 */
void mglPushMatrix()
{
}

/**
 * Pop the top matrix from the stack for the current matrix
 * mode.
 */
void mglPopMatrix()
{
}

/**
 * Replace the current matrix with the identity.
 */
void mglLoadIdentity()
{
}

/**
 * Replace the current matrix with an arbitrary 4x4 matrix,
 * specified in column-major order.  That is, the matrix
 * is stored as:
 *
 *   ( a0  a4  a8  a12 )
 *   ( a1  a5  a9  a13 )
 *   ( a2  a6  a10 a14 )
 *   ( a3  a7  a11 a15 )
 *
 * where ai is the i'th entry of the array.
 */
void mglLoadMatrix(const MGLfloat *matrix)
{
}

/**
 * Multiply the current matrix by an arbitrary 4x4 matrix,
 * specified in column-major order.  That is, the matrix
 * is stored as:
 *
 *   ( a0  a4  a8  a12 )
 *   ( a1  a5  a9  a13 )
 *   ( a2  a6  a10 a14 )
 *   ( a3  a7  a11 a15 )
 *
 * where ai is the i'th entry of the array.
 */
void mglMultMatrix(const MGLfloat *matrix)
{
}

/**
 * Multiply the current matrix by the translation matrix
 * for the translation vector given by (x, y, z).
 */
void mglTranslate(MGLfloat x,
                  MGLfloat y,
                  MGLfloat z)
{
}

/**
 * Multiply the current matrix by the rotation matrix
 * for a rotation of (angle) degrees about the vector
 * from the origin to the point (x, y, z).
 */
void mglRotate(MGLfloat angle,
               MGLfloat x,
               MGLfloat y,
               MGLfloat z)
{
}

/**
 * Multiply the current matrix by the scale matrix
 * for the given scale factors.
 */
void mglScale(MGLfloat x,
              MGLfloat y,
              MGLfloat z)
{
}

/**
 * Multiply the current matrix by the perspective matrix
 * with the given clipping plane coordinates.
 */
void mglFrustum(MGLfloat left,
                MGLfloat right,
                MGLfloat bottom,
                MGLfloat top,
                MGLfloat near,
                MGLfloat far)
{
}

/**
 * Multiply the current matrix by the orthographic matrix
 * with the given clipping plane coordinates.
 */
void mglOrtho(MGLfloat left,
              MGLfloat right,
              MGLfloat bottom,
              MGLfloat top,
              MGLfloat near,
              MGLfloat far)
{
}

/**
 * Set the current color for drawn shapes.
 */
void mglColor(MGLbyte red,
              MGLbyte green,
              MGLbyte blue)
{
}
